//
// CalSetter.cc
//
// Authors:  Richard Saxton (VEGA/ESA)
//           ESA XMM SOC
//
// Description: Reads keywords from a file and sets Cal state variables
//
/* Sas defs */
#include "errstr.h"
#include "Xmm.h"
#include "Dal.h"
#include "CalServer.h"
#include "ExceptionErrorHandler.h"

/* Other package defs */

/* Local defs */
#include "CalSetter.h"

/* Constructor */
CalSetter::CalSetter()
{
  PROC;
  ENDPROC;
}

/* Destructor */
CalSetter:: ~CalSetter()
{
  PROC;
  ENDPROC;
}

void CalSetter::fromImage(const string & infile)
{
  PROC;

   /* Open the dataset with read access */
    set = dataSetServer->open( infile, DataSet::Read );

   /* Set the instrument */
    string instr;

    if (set->hasAttribute("INSTRUME"))
     {
       instr = set->attribute("INSTRUME")->asString();
     }
    else
     {
       errstr
        << "Could not find an attribute INSTRUME in image"
        << "assuming the instrument is MOS1"
        << Sas::warning("NoInstrument");

      instr="EMOS1";
     }
    calServer.setInstrument(Xmm::toInstrument(instr));
    
   /* Set the window mode 
    if (set->hasAttribute("SUBMODE"))
     {
       _windowMode = getWindowMode( set->attribute("SUBMODE")->
                                                   asString() );
     } 
    else {
     _windowMode = getWindowMode(" ");
    }

    calServer.state()->mode()->set(_windowMode);
   */

   /* Switch off randomisation */
    calServer.state()->randomize()->set(false); 

   /* Set the CCD node to a default of 1 */
    calServer.state()->ccd()->set(1);

  ENDPROC;

}

/* Method to find the observing window mode */
Xmm::Mode CalSetter::getWindowMode(string submode) const
{
  PROC;

      ExceptionErrorHandler local;
      ErrorHandler* oldHandler = errHandler.handler(&local);
      Xmm::Mode windowMode;
      try {
            windowMode = Xmm::toMode(submode);
      } catch(SasException::Error) { 
             errstr<<"Submode not valid or missing - using FullWindow"
                   <<Sas::warning("UnknownModeString");
        windowMode = Xmm::PRIME_FULL_WINDOW; // d/f
      }
      errHandler.handler(oldHandler);

  RETURN windowMode;
}

/* Method to set the Encircled energy fraction PSF mode */
void CalSetter::setAccuracyLevel(AccuracyLevel mode) const
{
  PROC;

     calServer.state()->accuracyLevel()->set(mode);

  ENDPROC;
}

