//
// Module: EregionAnalyse
// Author: Richard Saxton 
//
// Description: Main class for calculating optimum image region
//
// Modifications: See ChangeLog
//

#ifndef EregionAnalyse_H
#define EregionAnalyse_H

/* General classes */
#include <iostream>

/* SAS-specific classes */
#include <Tracer.h>
#include <metatask.h>
#include <errstr.h>
#include <param.h>
#include <Dal.h>
#include <Xmm.h>
#include "CalServer.h"
#include "STime.h"

/* Local Classes */
#include "ImageRegion.h"
#include "CalSetter.h"


/****************************************************
 *
 * class EregionAnalyse
 *
 ***************************************************/

class EregionAnalyse
{
public:

  /* Constructor */
  EregionAnalyse();

  /* Destructor */
  ~EregionAnalyse();

  /* Define data structure */
  struct SOURCE {
      double bkgsub_counts,              // Source counts after bkgnd subt
             bkgsub_counts_error,        // Error on subtracted src counts
             bkgsub_count_rate,          // Bckgnd subtracted count rate
             bkgsub_count_rate_error,    // Error on subtracted count rate
             ee,                         // Applied encircled energy correction
             upplim;                     // Upper limit on source counts
  };

  /* Method to calculate the optimum radius */
  double optimumRadius(double src_cnts, double inradius,
                     double bck_flux,
                     double energy, double offax,
                     double phi);

  /* Method to calculate the optimum ellipse */
  FEllipse * optimumEllipse(double src_cnts,
                     double sboxrad,
                     double bck_flux,
                     double energy, double theta,
                     double phi, 
                     CoordPair centre, double orient);

  /* Calculates the background subtracted source count rate and error */
  void calcSrcCountRate(ImageRegion * srcreg,
                     ImageRegion * bckreg,
                     double bck_flux,
                     bool backgnd_region_set,
                     double energy, double exposure, double cl);

  /* Calculate the upper limit */
  double calcUpplim(double n, double b, double cl);

  /* Make a copy of the exposure map and add in REF keywords if necessary */
  const string expfile_copy(const string & exposurefile,
                                            const string & imagefile);

//  const string EregionAnalyse::getPostFix();

private:


  /* Member fields */
  SOURCE _source;
  ImageRegion * srcreg;
  ImageRegion * bckreg;
  ImageRegion * expreg;
  Psf *  _psf;
  CalSetter * _calset;
  string  _postfix;

};

#endif

