//
// Module: UpperLimitSolver
// Author: Richard Saxton 
//
// Description: Creates an ARF using arfgen
//
// Modifications: See ChangeLog
//
#ifndef UpperLimitSolver_H
#define UpperLimitSolver_H

/* General classes */
#include <iostream>
#include<cmath>
#include<vector>
#include<utility>

/* SAS-specific classes */
#include <Tracer.h>
#include <metatask.h>
#include <errstr.h>
#include <param.h>

/* Local Classes */

/****************************************************
 *
 * class UpperLimitSolver
 *
 ***************************************************/

class UpperLimitSolver
{
public:

  /* Constructor */
  UpperLimitSolver(long int n, double b);

  /* Destructor */
  ~UpperLimitSolver();

  /* The bayesian constant */
  void Const_C();

  /* The Bayesian calculation */
  std::pair<double, double> bayesian(const double CL);

  /* Calculation of factorial */
  double factorial(const double n);

private:

  /* The Bayesian calculation */
  std::pair<double, double> Bayesian_Confidence_Limits(
                                                     const double CL);

  /* Calculates the Bayesian posterior probability function */
  double ppdf(const double S);

  double B;
  long int N;

  double C;

};

#endif

